@@ -45,19 +45,21 @@ typedef enum {
 } TrackerDataOntologyError;
 
 GQuark   tracker_data_ontology_error_quark           (void);
-gboolean tracker_data_manager_init                   (TrackerDBManagerFlags  flags,
-                                                      const gchar          **test_schema,
-                                                      gboolean              *first_time,
-                                                      gboolean               journal_check,
-                                                      guint                  select_cache_size,
-                                                      guint                  update_cache_size,
-                                                      TrackerBusyCallback    busy_callback,
-                                                      gpointer               busy_user_data,
-                                                      const gchar           *busy_operation);
+gboolean tracker_data_manager_init                   (TrackerDBManagerFlags   flags,
+                                                      const gchar           **test_schema,
+                                                      gboolean               *first_time,
+                                                      gboolean                journal_check,
+                                                      guint                   select_cache_size,
+                                                      guint                   update_cache_size,
+                                                      TrackerBusyCallback     busy_callback,
+                                                      gpointer                busy_user_data,
+                                                      const gchar            *busy_operation,
+                                                      GError                **error);
 void     tracker_data_manager_shutdown               (void);
-gboolean tracker_data_manager_reload                 (TrackerBusyCallback    busy_callback,
-                                                      gpointer               busy_user_data,
-                                                      const gchar           *busy_operation);
+gboolean tracker_data_manager_reload                 (TrackerBusyCallback     busy_callback,
+                                                      gpointer                busy_user_data,
+                                                      const gchar            *busy_operation,
+                                                      GError                **error);
 
 G_END_DECLS
 
