@@ -28,6 +28,8 @@
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontologies.h>
 
+#include <libtracker-miner/tracker-miner-common.h>
+
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
@@ -151,6 +153,8 @@ static TrackerDataUpdateBuffer update_buffer;
 static TrackerDataUpdateBufferResource *resource_buffer;
 static TrackerDataBlankBuffer blank_buffer;
 static time_t resource_time = 0;
+static gint transaction_modseq = 0;
+static gboolean has_persistent = TRUE;
 
 static GPtrArray *insert_callbacks = NULL;
 static GPtrArray *delete_callbacks = NULL;
@@ -158,7 +162,6 @@ static GPtrArray *commit_callbacks = NULL;
 static GPtrArray *rollback_callbacks = NULL;
 static gint max_service_id = 0;
 static gint max_ontology_id = 0;
-static gint max_modseq = 0;
 
 static gint         ensure_resource_id      (const gchar      *uri,
                                              gboolean         *create);
@@ -413,14 +416,6 @@ tracker_data_update_get_new_service_id (void)
 	}
 }
 
-void
-tracker_data_update_shutdown (void)
-{
-	max_service_id = 0;
-	max_ontology_id = 0;
-	max_modseq = 0;
-}
-
 static gint
 tracker_data_update_get_next_modseq (void)
 {
@@ -428,10 +423,7 @@ tracker_data_update_get_next_modseq (void)
 	TrackerDBInterface *temp_iface;
 	TrackerDBStatement *stmt;
 	GError             *error = NULL;
-
-	if (G_LIKELY (max_modseq != 0)) {
-		return ++max_modseq;
-	}
+	gint                max_modseq = 0;
 
 	temp_iface = tracker_db_manager_get_db_interface ();
 
@@ -459,6 +451,28 @@ tracker_data_update_get_next_modseq (void)
 	return ++max_modseq;
 }
 
+void
+tracker_data_update_shutdown (void)
+{
+	max_service_id = 0;
+	max_ontology_id = 0;
+	transaction_modseq = 0;
+}
+
+static gint
+get_transaction_modseq (void)
+{
+	if (G_UNLIKELY (transaction_modseq == 0)) {
+		transaction_modseq = tracker_data_update_get_next_modseq ();
+	}
+
+	/* Always use 1 for ontology transactions */
+	if (in_ontology_transaction) {
+		return 1;
+	}
+
+	return transaction_modseq;
+}
 
 static TrackerDataUpdateBufferTable *
 cache_table_new (gboolean multiple_values)
@@ -502,7 +516,7 @@ cache_ensure_table (const gchar *table_name,
 		resource_buffer->modified = TRUE;
 
 		g_value_init (&gvalue, G_TYPE_INT64);
-		g_value_set_int64 (&gvalue, tracker_data_update_get_next_modseq ());
+		g_value_set_int64 (&gvalue, get_transaction_modseq ());
 		cache_insert_value ("rdfs:Resource", "tracker:modified", TRUE, &gvalue,
 		                    0,
 		                    FALSE, FALSE, FALSE);
@@ -887,7 +901,7 @@ tracker_data_resource_buffer_flush (GError **error)
 				if (strcmp (table_name, "rdfs:Resource") == 0) {
 					g_warn_if_fail	(resource_time != 0);
 					tracker_db_statement_bind_int (stmt, 1, (gint64) resource_time);
-					tracker_db_statement_bind_int (stmt, 2, tracker_data_update_get_next_modseq ());
+					tracker_db_statement_bind_int (stmt, 2, get_transaction_modseq ());
 					param = 3;
 				} else {
 					param = 1;
@@ -2014,6 +2028,8 @@ tracker_data_delete_statement (const gchar  *graph,
 	if (object && g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
 		class = tracker_ontologies_get_class_by_uri (object);
 		if (class != NULL) {
+			has_persistent = TRUE;
+
 			if (!in_journal_replay) {
 				tracker_db_journal_append_delete_statement_id (
 				       (graph != NULL ? query_resource_id (graph) : 0),
@@ -2033,6 +2049,10 @@ tracker_data_delete_statement (const gchar  *graph,
 
 		field = tracker_ontologies_get_property_by_uri (predicate);
 		if (field != NULL) {
+			if (!tracker_property_get_transient (field)) {
+				has_persistent = TRUE;
+			}
+
 			change = delete_metadata_decomposed (field, object, 0, error);
 			if (!in_journal_replay && change && !tracker_property_get_transient (field)) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
@@ -2052,10 +2072,22 @@ tracker_data_delete_statement (const gchar  *graph,
 					object_id = 0;
 					tried = TRUE;
 
-					tracker_db_journal_append_delete_statement (graph_id,
-					                                            resource_buffer->id,
-					                                            pred_id,
-					                                            object);
+					if (!tracker_property_get_force_journal (field) &&
+					    g_strcmp0 (graph, TRACKER_MINER_FS_GRAPH_URN) == 0) {
+						/* do not journal this statement extracted from filesystem */
+						TrackerProperty *damaged;
+
+						damaged = tracker_ontologies_get_property_by_uri (TRACKER_TRACKER_PREFIX "damaged");
+						tracker_db_journal_append_insert_statement (graph_id,
+										            resource_buffer->id,
+										            tracker_property_get_id (damaged),
+										            "true");
+					} else {
+						tracker_db_journal_append_delete_statement (graph_id,
+						                                            resource_buffer->id,
+						                                            pred_id,
+						                                            object);
+					}
 				}
 			}
 		} else {
@@ -2199,6 +2231,10 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		prop_id = tracker_property_get_id (property);
 	}
 
+	if (!tracker_property_get_transient (property)) {
+		has_persistent = TRUE;
+	}
+
 	/* subjects and objects starting with `:' are anonymous blank nodes */
 	if (g_str_has_prefix (object, ":")) {
 		/* anonymous blank node used as object in a statement */
@@ -2333,6 +2369,10 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 		pred_id = tracker_property_get_id (property);
 	}
 
+	if (!tracker_property_get_transient (property)) {
+		has_persistent = TRUE;
+	}
+
 	if (!tracker_data_insert_statement_common (graph, subject, predicate, object, &actual_error)) {
 		if (actual_error) {
 			g_propagate_error (error, actual_error);
@@ -2373,10 +2413,22 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
 			pred_id = (pred_id != 0) ? pred_id : tracker_data_query_resource_id (predicate);
 		}
-		tracker_db_journal_append_insert_statement (graph_id,
-		                                            resource_buffer->id,
-		                                            pred_id,
-		                                            object);
+		if (!tracker_property_get_force_journal (property) &&
+		    g_strcmp0 (graph, TRACKER_MINER_FS_GRAPH_URN) == 0) {
+			/* do not journal this statement extracted from filesystem */
+			TrackerProperty *damaged;
+
+			damaged = tracker_ontologies_get_property_by_uri (TRACKER_TRACKER_PREFIX "damaged");
+			tracker_db_journal_append_insert_statement (graph_id,
+				                                    resource_buffer->id,
+				                                    tracker_property_get_id (damaged),
+				                                    "true");
+		} else {
+			tracker_db_journal_append_insert_statement (graph_id,
+				                                    resource_buffer->id,
+				                                    pred_id,
+				                                    object);
+		}
 	}
 }
 
@@ -2395,6 +2447,8 @@ tracker_data_begin_transaction (GError **error)
 
 	resource_time = time (NULL);
 
+	has_persistent = FALSE;
+
 	if (update_buffer.resource_cache == NULL) {
 		update_buffer.resource_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 		/* used for normal transactions */
@@ -2458,13 +2512,25 @@ tracker_data_commit_transaction (GError **error)
 		return;
 	}
 
-	in_transaction = FALSE;
-	in_ontology_transaction = FALSE;
+	get_transaction_modseq ();
+	if (has_persistent && !in_ontology_transaction) {
+		transaction_modseq++;
+	}
 
 	if (!in_journal_replay) {
-		tracker_db_journal_commit_db_transaction ();
+		if (has_persistent || in_ontology_transaction) {
+			tracker_db_journal_commit_db_transaction ();
+		} else {
+			/* If we only had transient properties, then we must not write
+			 * anything to the journal. So we roll it back, but only the
+			 * journal's part. */
+			tracker_db_journal_rollback_transaction ();
+		}
 	}
+
 	resource_time = 0;
+	in_transaction = FALSE;
+	in_ontology_transaction = FALSE;
 
 	if (update_buffer.class_counts) {
 		/* successful transaction, no need to rollback class counts,
