@@ -44,15 +44,13 @@
 #define DEFAULT_INITIAL_SLEEP                    15       /* 0->1000 */
 #define DEFAULT_ENABLE_MONITORS                  TRUE
 #define DEFAULT_THROTTLE                         0        /* 0->20 */
-#define DEFAULT_SCAN_TIMEOUT                     0        /* 0->1000 */
-#define DEFAULT_CACHE_TIMEOUT                    60       /* 0->1000 */
-#define DEFAULT_INDEX_MOUNTED_DIRECTORIES        TRUE
 #define DEFAULT_INDEX_REMOVABLE_DEVICES          TRUE
 #define DEFAULT_INDEX_OPTICAL_DISCS              FALSE
 #define DEFAULT_INDEX_ON_BATTERY                 FALSE
 #define DEFAULT_INDEX_ON_BATTERY_FIRST_TIME      TRUE
 #define DEFAULT_LOW_DISK_SPACE_LIMIT             1        /* 0->100 / -1 */
-#define DEFAULT_CRAWLING_INTERVAL                0        /* 0->365 / -1 */
+#define DEFAULT_CRAWLING_INTERVAL                -1       /* 0->365 / -1 / -2 */
+#define DEFAULT_REMOVABLE_DAYS_THRESHOLD         3        /* 1->365 / 0  */
 
 typedef struct {
 	/* General */
@@ -61,14 +59,11 @@ typedef struct {
 
 	/* Monitors */
 	gboolean  enable_monitors;
-	gint      scan_timeout;
-	gint      cache_timeout;
 
 	/* Indexing */
 	gint      throttle;
 	gboolean  index_on_battery;
 	gboolean  index_on_battery_first_time;
-	gboolean  index_mounted_directories;
 	gboolean  index_removable_devices;
 	gboolean  index_optical_discs;
 	gint      low_disk_space_limit;
@@ -80,6 +75,7 @@ typedef struct {
 	GSList   *ignored_directories_with_content;
 	GSList   *ignored_files;
 	gint	  crawling_interval;
+	gint      removable_days_threshold;
 
 	/* Convenience data */
 	GSList   *ignored_directory_patterns;
@@ -122,14 +118,11 @@ enum {
 
 	/* Monitors */
 	PROP_ENABLE_MONITORS,
-	PROP_SCAN_TIMEOUT,
-	PROP_CACHE_TIMEOUT,
 
 	/* Indexing */
 	PROP_THROTTLE,
 	PROP_INDEX_ON_BATTERY,
 	PROP_INDEX_ON_BATTERY_FIRST_TIME,
-	PROP_INDEX_MOUNTED_DIRECTORIES,
 	PROP_INDEX_REMOVABLE_DEVICES,
 	PROP_INDEX_OPTICAL_DISCS,
 	PROP_LOW_DISK_SPACE_LIMIT,
@@ -138,7 +131,8 @@ enum {
 	PROP_IGNORED_DIRECTORIES,
 	PROP_IGNORED_DIRECTORIES_WITH_CONTENT,
 	PROP_IGNORED_FILES,
-	PROP_CRAWLING_INTERVAL
+	PROP_CRAWLING_INTERVAL,
+	PROP_REMOVABLE_DAYS_THRESHOLD
 };
 
 static ObjectToKeyFile conversions[] = {
@@ -147,13 +141,10 @@ static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_INT,     "initial-sleep",                    GROUP_GENERAL,  "InitialSleep"              },
 	/* Monitors */
 	{ G_TYPE_BOOLEAN, "enable-monitors",                  GROUP_MONITORS, "EnableMonitors"            },
-	{ G_TYPE_INT,     "scan-timeout",                     GROUP_MONITORS, "ScanTimeout"               },
-	{ G_TYPE_INT,     "cache-timeout",                    GROUP_MONITORS, "CacheTimeout"              },
 	/* Indexing */
 	{ G_TYPE_INT,     "throttle",                         GROUP_INDEXING, "Throttle"                  },
 	{ G_TYPE_BOOLEAN, "index-on-battery",                 GROUP_INDEXING, "IndexOnBattery"            },
 	{ G_TYPE_BOOLEAN, "index-on-battery-first-time",      GROUP_INDEXING, "IndexOnBatteryFirstTime"   },
-	{ G_TYPE_BOOLEAN, "index-mounted-directories",        GROUP_INDEXING, "IndexMountedDirectories"   },
 	{ G_TYPE_BOOLEAN, "index-removable-devices",          GROUP_INDEXING, "IndexRemovableMedia"       },
 	{ G_TYPE_BOOLEAN, "index-optical-discs",              GROUP_INDEXING, "IndexOpticalDiscs"         },
 	{ G_TYPE_INT,     "low-disk-space-limit",             GROUP_INDEXING, "LowDiskSpaceLimit"         },
@@ -162,7 +153,8 @@ static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_POINTER, "ignored-directories",              GROUP_INDEXING, "IgnoredDirectories"        },
 	{ G_TYPE_POINTER, "ignored-directories-with-content", GROUP_INDEXING, "IgnoredDirectoriesWithContent" },
 	{ G_TYPE_POINTER, "ignored-files",                    GROUP_INDEXING, "IgnoredFiles"              },
-	{ G_TYPE_INT,	  "crawling-interval",		      GROUP_INDEXING, "CrawlingInterval"	  }
+	{ G_TYPE_INT,	  "crawling-interval",                GROUP_INDEXING, "CrawlingInterval"          },
+	{ G_TYPE_INT,	  "removable-days-threshold",         GROUP_INDEXING, "RemovableDaysThreshold"    }
 };
 
 G_DEFINE_TYPE (TrackerConfig, tracker_config, TRACKER_TYPE_CONFIG_FILE);
@@ -208,24 +200,6 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                       " Set to false to completely disable any monitoring",
 	                                                       DEFAULT_ENABLE_MONITORS,
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (object_class,
-	                                 PROP_SCAN_TIMEOUT,
-	                                 g_param_spec_int ("scan-timeout",
-	                                                   "Scan Timeout",
-	                                                   " Time in seconds between same events to prevent flooding (0->1000)",
-	                                                   0,
-	                                                   1000,
-	                                                   DEFAULT_SCAN_TIMEOUT,
-	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (object_class,
-	                                 PROP_CACHE_TIMEOUT,
-	                                 g_param_spec_int ("cache-timeout",
-	                                                   "Scan Timeout",
-	                                                   " Time in seconds for events to be cached (0->1000)",
-	                                                   0,
-	                                                   1000,
-	                                                   DEFAULT_CACHE_TIMEOUT,
-	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/* Indexing */
 	g_object_class_install_property (object_class,
@@ -251,14 +225,6 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                       " Set to true to index while running on battery for the first time only",
 	                                                       DEFAULT_INDEX_ON_BATTERY_FIRST_TIME,
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (object_class,
-	                                 PROP_INDEX_MOUNTED_DIRECTORIES,
-	                                 g_param_spec_boolean ("index-mounted-directories",
-	                                                       "Index mounted directories",
-	                                                       " Set to true to enable traversing mounted directories on other file systems\n"
-	                                                       " (this excludes removable devices)",
-	                                                       DEFAULT_INDEX_MOUNTED_DIRECTORIES,
-	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 	                                 PROP_INDEX_REMOVABLE_DEVICES,
 	                                 g_param_spec_boolean ("index-removable-devices",
@@ -327,16 +293,29 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                       " List of files to NOT index (separator=;)",
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
-                                         PROP_CRAWLING_INTERVAL,
+	                                 PROP_CRAWLING_INTERVAL,
 	                                 g_param_spec_int ("crawling-interval",
 	                                                   "Crawling interval",
-                                                           " Interval in days to check the filesystem is up to date in the database."
-                                                           " If set to 0, crawling always occurs on startup, if -1 crawling is"
-                                                           " disabled entirely. Maximum is 365.",
-	                                                   -1,
+	                                                   " Interval in days to check the filesystem is up to date in the database,"
+	                                                   " maximum is 365, default is -1.\n"
+	                                                   "   -2 = crawling is disabled entirely\n"
+	                                                   "   -1 = crawling *may* occur on startup (if not cleanly shutdown)\n"
+	                                                   "    0 = crawling is forced",
+	                                                   -2,
 	                                                   365,
 	                                                   DEFAULT_CRAWLING_INTERVAL,
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (object_class,
+	                                 PROP_REMOVABLE_DAYS_THRESHOLD,
+	                                 g_param_spec_int ("removable-days-threshold",
+	                                                   "Removable days threshold",
+	                                                   " Threshold in days after which files from removables devices"
+	                                                   " will be removed from database if not mounted. 0 means never, "
+	                                                   " maximum is 365.",
+	                                                   0,
+	                                                   365,
+	                                                   DEFAULT_REMOVABLE_DAYS_THRESHOLD,
+	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_type_class_add_private (object_class, sizeof (TrackerConfigPrivate));
 }
@@ -368,14 +347,6 @@ config_set_property (GObject      *object,
 		tracker_config_set_enable_monitors (TRACKER_CONFIG (object),
 		                                    g_value_get_boolean (value));
 		break;
-	case PROP_SCAN_TIMEOUT:
-		tracker_config_set_scan_timeout (TRACKER_CONFIG (object),
-		                                 g_value_get_int (value));
-		break;
-	case PROP_CACHE_TIMEOUT:
-		tracker_config_set_cache_timeout (TRACKER_CONFIG (object),
-		                                  g_value_get_int (value));
-		break;
 
 		/* Indexing */
 	case PROP_THROTTLE:
@@ -390,10 +361,6 @@ config_set_property (GObject      *object,
 		tracker_config_set_index_on_battery_first_time (TRACKER_CONFIG (object),
 		                                                g_value_get_boolean (value));
 		break;
-	case PROP_INDEX_MOUNTED_DIRECTORIES:
-		tracker_config_set_index_mounted_directories (TRACKER_CONFIG (object),
-		                                              g_value_get_boolean (value));
-		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		tracker_config_set_index_removable_devices (TRACKER_CONFIG (object),
 		                                            g_value_get_boolean (value));
@@ -430,7 +397,10 @@ config_set_property (GObject      *object,
 		tracker_config_set_crawling_interval (TRACKER_CONFIG (object),
 		                                      g_value_get_int (value));
 		break;
-
+	case PROP_REMOVABLE_DAYS_THRESHOLD:
+		tracker_config_set_removable_days_threshold (TRACKER_CONFIG (object),
+		                                             g_value_get_int (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -460,12 +430,6 @@ config_get_property (GObject    *object,
 	case PROP_ENABLE_MONITORS:
 		g_value_set_boolean (value, priv->enable_monitors);
 		break;
-	case PROP_SCAN_TIMEOUT:
-		g_value_set_int (value, priv->scan_timeout);
-		break;
-	case PROP_CACHE_TIMEOUT:
-		g_value_set_int (value, priv->cache_timeout);
-		break;
 
 		/* Indexing */
 	case PROP_THROTTLE:
@@ -477,9 +441,6 @@ config_get_property (GObject    *object,
 	case PROP_INDEX_ON_BATTERY_FIRST_TIME:
 		g_value_set_boolean (value, priv->index_on_battery_first_time);
 		break;
-	case PROP_INDEX_MOUNTED_DIRECTORIES:
-		g_value_set_boolean (value, priv->index_mounted_directories);
-		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		g_value_set_boolean (value, priv->index_removable_devices);
 		break;
@@ -507,6 +468,9 @@ config_get_property (GObject    *object,
 	case PROP_CRAWLING_INTERVAL:
 		g_value_set_int (value, priv->crawling_interval);
 		break;
+	case PROP_REMOVABLE_DAYS_THRESHOLD:
+		g_value_set_int (value, priv->removable_days_threshold);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -864,9 +828,9 @@ config_load (TrackerConfig *config,
 
 	file = TRACKER_CONFIG_FILE (config);
 
-        if (use_defaults) {
-                config_create_with_defaults (config, file->key_file, FALSE);
-        }
+	if (use_defaults) {
+		config_create_with_defaults (config, file->key_file, FALSE);
+	}
 
 	if (!file->file_exists) {
 		tracker_config_file_save (file);
@@ -958,7 +922,7 @@ config_load (TrackerConfig *config,
 
 			if (!equal) {
 				g_object_set (config, conversions[i].property, new_dirs, NULL);
-                        }
+			}
 
 			g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
 			g_slist_free (new_dirs);
@@ -1128,30 +1092,6 @@ tracker_config_get_enable_monitors (TrackerConfig *config)
 	return priv->enable_monitors;
 }
 
-gint
-tracker_config_get_scan_timeout (TrackerConfig *config)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_SCAN_TIMEOUT);
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	return priv->scan_timeout;
-}
-
-gint
-tracker_config_get_cache_timeout (TrackerConfig *config)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_CACHE_TIMEOUT);
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	return priv->cache_timeout;
-}
-
 gint
 tracker_config_get_throttle (TrackerConfig *config)
 {
@@ -1188,18 +1128,6 @@ tracker_config_get_index_on_battery_first_time (TrackerConfig *config)
 	return priv->index_on_battery_first_time;
 }
 
-gboolean
-tracker_config_get_index_mounted_directories (TrackerConfig *config)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_INDEX_MOUNTED_DIRECTORIES);
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	return priv->index_mounted_directories;
-}
-
 gboolean
 tracker_config_get_index_removable_devices (TrackerConfig *config)
 {
@@ -1332,6 +1260,18 @@ tracker_config_get_crawling_interval (TrackerConfig *config)
 	return priv->crawling_interval;
 }
 
+gint
+tracker_config_get_removable_days_threshold (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), 0);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->removable_days_threshold;
+}
+
 void
 tracker_config_set_verbosity (TrackerConfig *config,
                               gint           value)
@@ -1382,42 +1322,6 @@ tracker_config_set_enable_monitors (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "enable-monitors");
 }
 
-void
-tracker_config_set_scan_timeout (TrackerConfig *config,
-                                 gint           value)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	if (!tracker_keyfile_object_validate_int (config, "scan-timeout", value)) {
-		return;
-	}
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	priv->scan_timeout = value;
-	g_object_notify (G_OBJECT (config), "scan-timeout");
-}
-
-void
-tracker_config_set_cache_timeout (TrackerConfig *config,
-                                  gint           value)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	if (!tracker_keyfile_object_validate_int (config, "cache-timeout", value)) {
-		return;
-	}
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	priv->cache_timeout = value;
-	g_object_notify (G_OBJECT (config), "cache-timeout");
-}
-
 void
 tracker_config_set_throttle (TrackerConfig *config,
                              gint           value)
@@ -1464,20 +1368,6 @@ tracker_config_set_index_on_battery_first_time (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "index-on-battery-first-time");
 }
 
-void
-tracker_config_set_index_mounted_directories (TrackerConfig *config,
-                                              gboolean       value)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	priv->index_mounted_directories = value;
-	g_object_notify (G_OBJECT (config), "index-mounted-directories");
-}
-
 void
 tracker_config_set_index_removable_devices (TrackerConfig *config,
                                             gboolean       value)
@@ -1538,8 +1428,8 @@ rebuild_filtered_lists (TrackerConfig *config)
 
 	if (priv->index_single_directories_unfiltered) {
 		priv->index_single_directories =
-		        tracker_path_list_filter_duplicates (priv->index_single_directories_unfiltered,
-		                                             ".", FALSE);
+			tracker_path_list_filter_duplicates (priv->index_single_directories_unfiltered,
+			                                     ".", FALSE);
 	}
 
 	if (!tracker_gslist_with_string_data_equal (old_list, priv->index_single_directories)) {
@@ -1574,7 +1464,7 @@ rebuild_filtered_lists (TrackerConfig *config)
 		new_list = g_slist_reverse (new_list);
 
 		priv->index_recursive_directories =
-		        tracker_path_list_filter_duplicates (new_list, ".", TRUE);
+			tracker_path_list_filter_duplicates (new_list, ".", TRUE);
 
 		g_slist_free (new_list);
 	}
@@ -1595,7 +1485,7 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 {
 	TrackerConfigPrivate *priv;
 	GSList *l;
-        gboolean equal;
+	gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1603,7 +1493,7 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 
 	l = priv->index_recursive_directories_unfiltered;
 
-        equal = tracker_gslist_with_string_data_equal (roots, l);
+	equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
 		priv->index_recursive_directories_unfiltered = NULL;
@@ -1615,11 +1505,11 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
-        if (equal) {
-                return;
-        }
+	if (equal) {
+		return;
+	}
 
-        rebuild_filtered_lists (config);
+	rebuild_filtered_lists (config);
 }
 
 void
@@ -1628,7 +1518,7 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 {
 	TrackerConfigPrivate *priv;
 	GSList *l;
-        gboolean equal;
+	gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1636,7 +1526,7 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 
 	l = priv->index_single_directories_unfiltered;
 
-        equal = tracker_gslist_with_string_data_equal (roots, l);
+	equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
 		priv->index_single_directories_unfiltered = NULL;
@@ -1648,11 +1538,11 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
-        if (equal) {
-                return;
-        }
+	if (equal) {
+		return;
+	}
 
-        rebuild_filtered_lists (config);
+	rebuild_filtered_lists (config);
 }
 
 void
@@ -1661,7 +1551,7 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 {
 	TrackerConfigPrivate *priv;
 	GSList *l;
-        gboolean equal;
+	gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1669,7 +1559,7 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 
 	l = priv->ignored_directories;
 
-        equal = tracker_gslist_with_string_data_equal (roots, l);
+	equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
 		priv->ignored_directories = NULL;
@@ -1681,9 +1571,9 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
-        if (equal) {
-                return;
-        }
+	if (equal) {
+		return;
+	}
 
 	/* Re-set up the GPatternSpec list */
 	config_set_ignored_directory_conveniences (config);
@@ -1697,7 +1587,7 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
 {
 	TrackerConfigPrivate *priv;
 	GSList *l;
-        gboolean equal;
+	gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1705,7 +1595,7 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
 
 	l = priv->ignored_directories_with_content;
 
-        equal = tracker_gslist_with_string_data_equal (roots, l);
+	equal = tracker_gslist_with_string_data_equal (roots, l);
 
 	if (!roots) {
 		priv->ignored_directories_with_content = NULL;
@@ -1717,9 +1607,9 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
-        if (equal) {
-                return;
-        }
+	if (equal) {
+		return;
+	}
 
 	g_object_notify (G_OBJECT (config), "ignored-directories-with-content");
 }
@@ -1730,7 +1620,7 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 {
 	TrackerConfigPrivate *priv;
 	GSList *l;
-        gboolean equal;
+	gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1738,7 +1628,7 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 
 	l = priv->ignored_files;
 
-        equal = tracker_gslist_with_string_data_equal (files, l);
+	equal = tracker_gslist_with_string_data_equal (files, l);
 
 	if (!files) {
 		priv->ignored_files = NULL;
@@ -1750,9 +1640,9 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
-        if (equal) {
-                return;
-        }
+	if (equal) {
+		return;
+	}
 
 	/* Re-set up the GPatternSpec list */
 	config_set_ignored_file_conveniences (config);
@@ -1778,6 +1668,24 @@ tracker_config_set_crawling_interval (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "crawling-interval");
 }
 
+void
+tracker_config_set_removable_days_threshold (TrackerConfig *config,
+                                             gint           value)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_CONFIG (config));
+
+	if (!tracker_keyfile_object_validate_int (config, "removable-days-threshold", value)) {
+		return;
+	}
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	priv->removable_days_threshold = value;
+	g_object_notify (G_OBJECT (config), "removable-days-threshold");
+}
+
 /*
  * Convenience functions
  */
