@@ -21,6 +21,8 @@
 
 #include <stdlib.h>
 
+#include <glib/gi18n.h>
+
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-sparql/tracker-sparql.h>
 
@@ -30,6 +32,12 @@
 #include "tracker-writeback-module.h"
 #include "tracker-marshal.h"
 
+/* Known exception for which linking with libtracker-miner isn't necessary,
+ * include is needed for the TRACKER_MINER_FS_GRAPH_URN define which is shared
+ * between tracker-writeback and tracker-miner-fs */
+
+#include "libtracker-miner/tracker-miner-common.h"
+
 #define TRACKER_WRITEBACK_CONSUMER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_WRITEBACK_CONSUMER, TrackerWritebackConsumerPrivate))
 
 #define TRACKER_SERVICE             "org.freedesktop.Tracker1"
@@ -184,7 +192,7 @@ sparql_query_cb (GObject      *object,
 	cursor = tracker_sparql_connection_query_finish (TRACKER_SPARQL_CONNECTION (object), result, &error);
 
 	if (!error) {
-		GPtrArray *results = g_ptr_array_new ();
+		GPtrArray *results = g_ptr_array_new_with_free_func ((GDestroyNotify) g_strfreev);
 		guint cols = tracker_sparql_cursor_get_n_columns (cursor);
 
 		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
@@ -223,11 +231,10 @@ sparql_query_cb (GObject      *object,
 					g_object_unref (writeback);
 				}
 			}
-			g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
 		} else {
 			g_message ("  No files qualify for updates");
 		}
-		g_ptr_array_free (results, TRUE);
+		g_ptr_array_unref (results);
 		g_object_unref (cursor);
 	} else {
 		g_message ("  No files qualify for updates (%s)", error->message);
@@ -297,12 +304,13 @@ rdf_types_to_uris_cb (GObject      *object,
 			goto trouble;
 		}
 
-		query = g_strdup_printf ("SELECT ?url '%s' ?predicate ?object {"
-		                         "  <%s> ?predicate ?object ; "
-		                         "       nie:url ?url ."
-		                         "  ?predicate tracker:writeback true "
-		                         "}",
-		                         subject, subject);
+		query = g_strdup_printf ("SELECT ?url '%s' ?predicate ?object { "
+		                            "<%s> a nfo:FileDataObject . "
+		                            "<%s> ?predicate ?object ; nie:url ?url . "
+		                            "?predicate tracker:writeback true . "
+		                            "FILTER (NOT EXISTS { GRAPH <"TRACKER_MINER_FS_GRAPH_URN"> "
+		                              "{ <%s> ?predicate ?object } }) } ",
+		                         subject, subject, subject, subject);
 
 		tracker_sparql_connection_query_async (priv->connection,
 		                                       query,
